/*
 * ERIKA Enterprise Basic - a tiny RTOS for small microcontrollers
 *
 * Copyright (C) 2002-2007  Evidence Srl
 *
 * This file is part of ERIKA Enterprise Basic.
 *
 * ERIKA Enterprise Basic is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 *
 * ERIKA Enterprise Basic is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License version 2 for more details.
 *
 * You should have received a copy of the GNU General Public License
 * version 2 along with ERIKA Enterprise Basic; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA.
 */

/*
 * Author: 2006 Paolo Gai
 * CVS: $Id: ee_mcu.h,v 1.6 2007/06/14 10:27:12 pj Exp $
 */

#include "mcu/microchip_dspic/inc/ee_mcuregs.h"

#ifndef __INCLUDE_MICROCHIP_DSPIC_MCU_H__
#define __INCLUDE_MICROCHIP_DSPIC_MCU_H__

/*************************************************************************
 Time handling
 *************************************************************************/

/*
 * Time handling on the dsPIC33 is made by using timer T8 and T9 as a
 * 32 bit register value to have a bigger lifetime.
 */

/* Time types
 *
 * This is the type used to store the circular timer. It is in the MCU
 * because it highly depends on the hardware timers which are present 
 * in a particular microcontroller version.
 *
 * There are two TIME types, an unsigned type, typically used to store
 * time values since they are read from a register which is
 * traditionally considered as unsigned, and a signed type, which is
 * used in the EDF comparisons.
 */

#ifndef EE_TIME
#define EE_TIME EE_UINT32
#endif

#ifndef EE_STIME
#define EE_STIME EE_INT32
#endif

/* compile these functions only if TMR8 is available on the
   particular microcontroller */
#ifdef __PIC30_HAS_TMR8__

/* This function initializes the timer T8 and T9 as freerunning.
   Timer T8 and T9 will be used to get the timing reference for the
   circular timer in EDF.
*/
#ifndef __PRIVATE_TIME_INIT__
__INLINE__ void __ALWAYS_INLINE__ EE_time_init(void)
{
  T8CON = 0;          /* Stops the Timer8 and reset control reg	*/
  T8CONbits.T32 = 1;  /* Set Timer8 in 32bit mode */
  TMR8  = 0;          /* Clear contents of the timer registers	*/
  TMR9  = 0;
  PR8   = 0xffff;     /* Load the Period registers wit the value 0xffffffff */
  PR9   = 0xffff;
  IFS3bits.T9IF  = 0; /* Clear the Timer9 interrupt status flag	*/
  IEC3bits.T9IE  = 0; /* Clear Timer9 interrupts		*/
  T8CONbits.TON  = 1; /* Start Timer8 with prescaler settings at 1:1
		       * and clock source set to the internal 
		       * instruction cycle			*/
}
#endif

/*
 * The function gets the current time by concatenating two timer values.
 *
 * Note: we must take care of the correctness of the value in case of
 * overflow!
 */
#ifndef __PRIVATE_HAL_GETTIME__
__INLINE__ EE_TIME __ALWAYS_INLINE__ EE_hal_gettime(void)
{
  union {
    struct { EE_UREG low, hi; } lowhi;
    EE_TIME time;
  } retvalue;
  
  // I split the reading to guarantee that in any case TMR8 will be
  // read before TMR9. In this way, the TMR9 hold register will be
  // read correctly!

  retvalue.lowhi.low = TMR8;
  retvalue.lowhi.hi = TMR9;
  return retvalue.time;
}
#endif

#else

/* If you are using EDF with a MCU without TMR8, then put a warning! */
#ifdef __EDF__
#ifndef __PRIVATE_TIME_INIT__
#warning Please provide a custom EE_time_init because the MCU you are using does not have TMR8!
#endif
#ifndef __PRIVATE_HAL_GETTIME__
#warning Please provide a custom EE_hal_gettime because the MCU you are using does not have TMR8!
#endif
#endif

#endif

#endif
